import os

REQUESTS_TIMEOUT = 120
BACKEND_BASE_URL = "http://localhost:8080"  # TODO: replace with dynamic API url
DEFAULT_CONFIG_DIRECTORY = os.path.expanduser("~/.beam")
DEFAULT_SYNCTHING_LOCAL_PORT = 32768
DEFAULT_SYNCTHING_REMOTE_PORT = 443
DEFAULT_CONFIG_CPU = 8
DEFAULT_CONFIG_MEMORY = "32Gi"
DEFAULT_CONFIG_GPU = 0
